/*
 * *** Please do not edit ! ***
 * @(#) CheckForConflict_4.cs
 * @procedure check_for_conflict/4 in dca2.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class CheckForConflict_4 : Predicate {
    static internal readonly Predicate dollar_fail_0 = new Predicates.dollar_fail_0();
    static internal readonly Predicate CheckForConflict_4_1 = new Predicates.CheckForConflict_4_1();
    static internal readonly Predicate CheckForConflict_4_2 = new Predicates.CheckForConflict_4_2();
    static internal readonly Predicate CheckForConflict_4_3 = new Predicates.CheckForConflict_4_3();
    static internal readonly Predicate CheckForConflict_4_lis = new Predicates.CheckForConflict_4_lis();
    static internal readonly Predicate CheckForConflict_4_var = new Predicates.CheckForConflict_4_var();

    public Term arg1, arg2, arg3, arg4;

    public CheckForConflict_4(Term a1, Term a2, Term a3, Term a4, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        this.cont = cont;
    }

    public CheckForConflict_4(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   CheckForConflict_4_var,
                                   dollar_fail_0,
                                   CheckForConflict_4_1,
                                   dollar_fail_0,
                                   CheckForConflict_4_lis
                                   );
    }

    public override int arity() { return 4; }

    public override string ToString() {
        return "check_for_conflict(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ")";
    }
}

sealed class CheckForConflict_4_var : CheckForConflict_4 {
    static internal readonly Predicate CheckForConflict_4_var_1 = new Predicates.CheckForConflict_4_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(CheckForConflict_4_1, CheckForConflict_4_var_1);
    }
}

sealed class CheckForConflict_4_var_1 : CheckForConflict_4 {
    static internal readonly Predicate CheckForConflict_4_var_2 = new Predicates.CheckForConflict_4_var_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(CheckForConflict_4_2, CheckForConflict_4_var_2);
    }
}

sealed class CheckForConflict_4_var_2 : CheckForConflict_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(CheckForConflict_4_3);
    }
}

sealed class CheckForConflict_4_lis : CheckForConflict_4 {
    static internal readonly Predicate CheckForConflict_4_lis_1 = new Predicates.CheckForConflict_4_lis_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(CheckForConflict_4_2, CheckForConflict_4_lis_1);
    }
}

sealed class CheckForConflict_4_lis_1 : CheckForConflict_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(CheckForConflict_4_3);
    }
}

sealed class CheckForConflict_4_1 : CheckForConflict_4 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a1, engine.trail) ) return engine.fail();
        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class CheckForConflict_4_2 : CheckForConflict_4 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a5 = ((ListTerm)a1).car;
            a6 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a5 = engine.makeVariable();
            a6 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a5, a6), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a2.isList() ){
            a7 = ((ListTerm)a2).car;
            a8 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a7 = engine.makeVariable();
            a8 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a7, a8), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isList() ){
            a9 = ((ListTerm)a3).car;
            a10 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a9 = engine.makeVariable();
            a10 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a9, a10), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.CheckForConflict_4(a6, a8, a10, a4, cont);
        p2 = new Predicates.dollar_unify_2(a5, a7, p1);
        return new Predicates.dollar_unify_2(a9, a4, p2);
    }
}

sealed class CheckForConflict_4_3 : CheckForConflict_4 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a5 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a5 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(engine.makeVariable(), a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a2.isList() ){
            a6 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a6 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(engine.makeVariable(), a6), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isList() ){
            a7 = ((ListTerm)a3).car;
            a8 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a7 = engine.makeVariable();
            a8 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a7, a8), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.CheckForConflict_4(a5, a6, a8, a4, cont);
        return new Predicates.dollar_dummyDca2__5_2(a7, a4, p1);
    }
}
}

