/*
 * *** Please do not edit ! ***
 * @(#) FilterConflicts_3.cs
 * @procedure filter_conflicts/3 in dca2.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class FilterConflicts_3 : Predicate, FcoPredicate {
    static internal readonly System.Type EntryCodeType = typeof( FilterConflicts_3 );
    static internal readonly Predicate FilterConflicts_3_1 = new Predicates.FilterConflicts_3_1();
    static internal readonly Predicate FilterConflicts_3_2 = new Predicates.FilterConflicts_3_2();
    static internal readonly Predicate FilterConflicts_3_3 = new Predicates.FilterConflicts_3_3();
    static internal readonly Predicate FilterConflicts_3_sub_1 = new Predicates.FilterConflicts_3_sub_1();

    public Term arg1, arg2, arg3;

    public FilterConflicts_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public FilterConflicts_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.SetEntryCode( EntryCodeType, this );
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(FilterConflicts_3_1, FilterConflicts_3_sub_1);
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "filter_conflicts(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class FilterConflicts_3_sub_1 : FilterConflicts_3 {
    static internal readonly Predicate FilterConflicts_3_sub_2 = new Predicates.FilterConflicts_3_sub_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(FilterConflicts_3_2, FilterConflicts_3_sub_2);
    }
}

sealed class FilterConflicts_3_sub_2 : FilterConflicts_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(FilterConflicts_3_3);
    }
}

sealed class FilterConflicts_3_1 : FilterConflicts_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class FilterConflicts_3_2 : FilterConflicts_3 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("/", 2);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10;
        Predicate p1, p2, p3, p4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a2.isList() ){
            a4 = ((ListTerm)a2).car;
            a5 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a4 = engine.makeVariable();
            a5 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a4, a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isList() ){
            if ( !a4.unify(((ListTerm)a3).car, engine.trail) )
                return engine.fail();
            a6 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a6 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a4, a6), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a7 = engine.makeVariable();
        a8 = engine.makeVariable();
        a9 = engine.makeVariable();
        Term[] h2 = {a8, a9};
        a10 = new StructureTerm(f1, h2);
        p1 = new Predicates.FilterConflicts_3(a1, a5, a6, cont);
        p2 = new Predicates.dollar_cut_1(a7, p1);
        p3 = new Predicates.Conflict_3(a10, a1, a4, p2);
        p4 = new Predicates.dollar_functor_3(a1, a8, a9, p3);
        return new Predicates.dollar_getLevel_1(a7, p4);
    }
}

sealed class FilterConflicts_3_3 : FilterConflicts_3 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a2.isList() ){
            a4 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a4 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(engine.makeVariable(), a4), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        engine.aregs[1] = a1;
        engine.aregs[2] = a4;
        engine.aregs[3] = a3;
        engine.cont = cont;
        return engine.GetEntryCode( EntryCodeType ).call( engine );
    }
}
}

