/*
 * *** Please do not edit ! ***
 * @(#) ResolvePredicate_4.cs
 * @procedure resolve_predicate/4 in dca2.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class ResolvePredicate_4 : Predicate {
    static internal readonly Predicate ResolvePredicate_4_1 = new Predicates.ResolvePredicate_4_1();
    static internal readonly Predicate ResolvePredicate_4_2 = new Predicates.ResolvePredicate_4_2();
    static internal readonly Predicate ResolvePredicate_4_sub_1 = new Predicates.ResolvePredicate_4_sub_1();

    public Term arg1, arg2, arg3, arg4;

    public ResolvePredicate_4(Term a1, Term a2, Term a3, Term a4, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        this.cont = cont;
    }

    public ResolvePredicate_4(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(ResolvePredicate_4_1, ResolvePredicate_4_sub_1);
    }

    public override int arity() { return 4; }

    public override string ToString() {
        return "resolve_predicate(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ")";
    }
}

sealed class ResolvePredicate_4_sub_1 : ResolvePredicate_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(ResolvePredicate_4_2);
    }
}

sealed class ResolvePredicate_4_1 : ResolvePredicate_4 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("global_call", 1);
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("fact", 2);
    static internal readonly SymbolTerm f5 = SymbolTerm.makeSymbol("resolve", 4);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9;
        Predicate p1, p2, p3, p4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        a5 = engine.makeVariable();
        Term[] h3 = {a1, a5};
        a7 = new StructureTerm(f2, h3);
        Term[] h4 = {a7};
        a6 = new StructureTerm(f1, h4);
        a8 = engine.makeVariable();
        Term[] h6 = {a8, a2, a3, a4};
        a9 = new StructureTerm(f5, h6);
        p1 = new Predicates.WaitFor_1(a4, cont);
        p2 = new Predicates.Fork_1(a9, p1);
        p3 = new Predicates.dollar_dummyDca2__4_1(a8, p2);
        p4 = new Predicates.Findall_3(a5, a6, a8, p3);
        return new Predicates.BacktrackableLock_1(a1, p4);
    }
}

sealed class ResolvePredicate_4_2 : ResolvePredicate_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.cont;
    }
}
}

