/*
 * *** Please do not edit ! ***
 * @(#) Resolve_4.cs
 * @procedure resolve/4 in dca2.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Resolve_4 : Predicate {
    static internal readonly Predicate dollar_fail_0 = new Predicates.dollar_fail_0();
    static internal readonly Predicate Resolve_4_1 = new Predicates.Resolve_4_1();
    static internal readonly Predicate Resolve_4_2 = new Predicates.Resolve_4_2();
    static internal readonly Predicate Resolve_4_var = new Predicates.Resolve_4_var();

    public Term arg1, arg2, arg3, arg4;

    public Resolve_4(Term a1, Term a2, Term a3, Term a4, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        this.cont = cont;
    }

    public Resolve_4(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   Resolve_4_var,
                                   dollar_fail_0,
                                   Resolve_4_1,
                                   dollar_fail_0,
                                   Resolve_4_2
                                   );
    }

    public override int arity() { return 4; }

    public override string ToString() {
        return "resolve(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ")";
    }
}

sealed class Resolve_4_var : Resolve_4 {
    static internal readonly Predicate Resolve_4_var_1 = new Predicates.Resolve_4_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Resolve_4_1, Resolve_4_var_1);
    }
}

sealed class Resolve_4_var_1 : Resolve_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Resolve_4_2);
    }
}

sealed class Resolve_4_1 : Resolve_4 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a1, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class Resolve_4_2 : Resolve_4 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8;
        Predicate p1, p2, p3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a5 = ((ListTerm)a1).car;
            a6 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a5 = engine.makeVariable();
            a6 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a5, a6), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a7 = engine.makeVariable();
        a8 = engine.makeVariable();
        p1 = new Predicates.Resolve_4(a8, a2, a3, a4, cont);
        p2 = new Predicates.dollar_dummyDca2__8_5(a5, a2, a3, a4, a7, p1);
        p3 = new Predicates.Setdiff_3(a6, a7, a8, p2);
        return new Predicates.FilterConflicts_3(a5, a6, a7, p3);
    }
}
}

