/*
 * *** Please do not edit ! ***
 * @(#) Setdiff_3.cs
 * @procedure setdiff/3 in dca2.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Setdiff_3 : Predicate {
    static internal readonly Predicate Setdiff_3_1 = new Predicates.Setdiff_3_1();
    static internal readonly Predicate Setdiff_3_2 = new Predicates.Setdiff_3_2();
    static internal readonly Predicate Setdiff_3_3 = new Predicates.Setdiff_3_3();
    static internal readonly Predicate Setdiff_3_var = new Predicates.Setdiff_3_var();

    public Term arg1, arg2, arg3;

    public Setdiff_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public Setdiff_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   Setdiff_3_var,
                                   Setdiff_3_1,
                                   Setdiff_3_1,
                                   Setdiff_3_1,
                                   Setdiff_3_var
                                   );
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "setdiff(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class Setdiff_3_var : Setdiff_3 {
    static internal readonly Predicate Setdiff_3_var_1 = new Predicates.Setdiff_3_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Setdiff_3_1, Setdiff_3_var_1);
    }
}

sealed class Setdiff_3_var_1 : Setdiff_3 {
    static internal readonly Predicate Setdiff_3_var_2 = new Predicates.Setdiff_3_var_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(Setdiff_3_2, Setdiff_3_var_2);
    }
}

sealed class Setdiff_3_var_2 : Setdiff_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Setdiff_3_3);
    }
}

sealed class Setdiff_3_1 : Setdiff_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        if ( !a1.unify(a3, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class Setdiff_3_2 : Setdiff_3 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a4 = ((ListTerm)a1).car;
            a5 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a4 = engine.makeVariable();
            a5 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a4, a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a2.isList() ){
            if ( !a4.unify(((ListTerm)a2).car, engine.trail) )
                return engine.fail();
            a6 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a6 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a4, a6), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a7 = engine.makeVariable();
        p1 = new Predicates.dollar_cut_1(a7, cont);
        p2 = new Predicates.Setdiff_3(a5, a6, a3, p1);
        return new Predicates.dollar_getLevel_1(a7, p2);
    }
}

sealed class Setdiff_3_3 : Setdiff_3 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a4 = ((ListTerm)a1).car;
            a5 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a4 = engine.makeVariable();
            a5 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a4, a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a2.isList() ){
            a6 = ((ListTerm)a2).car;
            a7 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a6 = engine.makeVariable();
            a7 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a6, a7), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isList() ){
            if ( !a4.unify(((ListTerm)a3).car, engine.trail) )
                return engine.fail();
            a8 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a8 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a4, a8), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a9 = new ListTerm(a6, a7);
        p1 = new Predicates.Setdiff_3(a5, a9, a8, cont);
        return new Predicates.dollar_dummyDca2__9_2(a4, a6, p1);
    }
}
}

