/*
 * *** Please do not edit ! ***
 * @(#) ListEqual_2.cs
 * @procedure list_equal/2 in search.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class ListEqual_2 : Predicate, FcoPredicate {
    static internal readonly System.Type EntryCodeType = typeof( ListEqual_2 );
    static internal readonly Predicate dollar_fail_0 = new Predicates.dollar_fail_0();
    static internal readonly Predicate ListEqual_2_1 = new Predicates.ListEqual_2_1();
    static internal readonly Predicate ListEqual_2_2 = new Predicates.ListEqual_2_2();
    static internal readonly Predicate ListEqual_2_var = new Predicates.ListEqual_2_var();

    public Term arg1, arg2;

    public ListEqual_2(Term a1, Term a2, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        this.cont = cont;
    }

    public ListEqual_2(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.SetEntryCode( EntryCodeType, this );
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   ListEqual_2_var,
                                   dollar_fail_0,
                                   ListEqual_2_1,
                                   dollar_fail_0,
                                   ListEqual_2_2
                                   );
    }

    public override int arity() { return 2; }

    public override string ToString() {
        return "list_equal(" + arg1 + ", " + arg2 + ")";
    }
}

sealed class ListEqual_2_var : ListEqual_2 {
    static internal readonly Predicate ListEqual_2_var_1 = new Predicates.ListEqual_2_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(ListEqual_2_1, ListEqual_2_var_1);
    }
}

sealed class ListEqual_2_var_1 : ListEqual_2 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(ListEqual_2_2);
    }
}

sealed class ListEqual_2_1 : ListEqual_2 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a1, engine.trail) ) return engine.fail();
        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        return cont;
    }
}

sealed class ListEqual_2_2 : ListEqual_2 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a3 = ((ListTerm)a1).car;
            a4 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a3 = engine.makeVariable();
            a4 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a3, a4), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a2.isList() ){
            if ( !a3.unify(((ListTerm)a2).car, engine.trail) )
                return engine.fail();
            a5 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a5 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a3, a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        engine.aregs[1] = a4;
        engine.aregs[2] = a5;
        engine.cont = cont;
        return engine.GetEntryCode( EntryCodeType ).call( engine );
    }
}
}

