/*
 * *** Please do not edit ! ***
 * @(#) Pulse_2.cs
 * @procedure pulse/2 in search.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Pulse_2 : Predicate {
    static internal readonly Predicate Pulse_2_1 = new Predicates.Pulse_2_1();
    static internal readonly Predicate Pulse_2_2 = new Predicates.Pulse_2_2();
    static internal readonly Predicate Pulse_2_3 = new Predicates.Pulse_2_3();
    static internal readonly Predicate Pulse_2_sub_1 = new Predicates.Pulse_2_sub_1();

    public Term arg1, arg2;

    public Pulse_2(Term a1, Term a2, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        this.cont = cont;
    }

    public Pulse_2(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(Pulse_2_1, Pulse_2_sub_1);
    }

    public override int arity() { return 2; }

    public override string ToString() {
        return "pulse(" + arg1 + ", " + arg2 + ")";
    }
}

sealed class Pulse_2_sub_1 : Pulse_2 {
    static internal readonly Predicate Pulse_2_sub_2 = new Predicates.Pulse_2_sub_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(Pulse_2_2, Pulse_2_sub_2);
    }
}

sealed class Pulse_2_sub_2 : Pulse_2 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Pulse_2_3);
    }
}

sealed class Pulse_2_1 : Pulse_2 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        p1 = new Predicates.dollar_fail_0(cont);
        p2 = new Predicates.IncCount_0(p1);
        return new Predicates.dollar_unify_2(a1, a2, p2);
    }
}

sealed class Pulse_2_2 : Pulse_2 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("keep_going");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        a3 = engine.makeVariable();
        p1 = new Predicates.dollar_cut_1(a3, cont);
        p2 = new Predicates.GlobalCall_1(s1, p1);
        return new Predicates.dollar_getLevel_1(a3, p2);
    }
}

sealed class Pulse_2_3 : Pulse_2 {

    public override Predicate exec( Prolog engine ) {
        Term a1, a2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        return new Predicates.dollar_abort_0(cont);
    }
}
}

