/*
 * *** Please do not edit ! ***
 * @(#) Truncate_4.cs
 * @procedure truncate/4 in search.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Truncate_4 : Predicate {
    static internal readonly Predicate Truncate_4_1 = new Predicates.Truncate_4_1();
    static internal readonly Predicate Truncate_4_2 = new Predicates.Truncate_4_2();
    static internal readonly Predicate Truncate_4_3 = new Predicates.Truncate_4_3();
    static internal readonly Predicate Truncate_4_con = new Predicates.Truncate_4_con();
    static internal readonly Predicate Truncate_4_lis = new Predicates.Truncate_4_lis();
    static internal readonly Predicate Truncate_4_var = new Predicates.Truncate_4_var();

    public Term arg1, arg2, arg3, arg4;

    public Truncate_4(Term a1, Term a2, Term a3, Term a4, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        this.cont = cont;
    }

    public Truncate_4(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   Truncate_4_var,
                                   Truncate_4_2,
                                   Truncate_4_con,
                                   Truncate_4_2,
                                   Truncate_4_lis
                                   );
    }

    public override int arity() { return 4; }

    public override string ToString() {
        return "truncate(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ")";
    }
}

sealed class Truncate_4_var : Truncate_4 {
    static internal readonly Predicate Truncate_4_var_1 = new Predicates.Truncate_4_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Truncate_4_1, Truncate_4_var_1);
    }
}

sealed class Truncate_4_var_1 : Truncate_4 {
    static internal readonly Predicate Truncate_4_var_2 = new Predicates.Truncate_4_var_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(Truncate_4_2, Truncate_4_var_2);
    }
}

sealed class Truncate_4_var_2 : Truncate_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Truncate_4_3);
    }
}

sealed class Truncate_4_con : Truncate_4 {
    static internal readonly Predicate Truncate_4_con_1 = new Predicates.Truncate_4_con_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Truncate_4_1, Truncate_4_con_1);
    }
}

sealed class Truncate_4_con_1 : Truncate_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Truncate_4_2);
    }
}

sealed class Truncate_4_lis : Truncate_4 {
    static internal readonly Predicate Truncate_4_lis_1 = new Predicates.Truncate_4_lis_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Truncate_4_2, Truncate_4_lis_1);
    }
}

sealed class Truncate_4_lis_1 : Truncate_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Truncate_4_3);
    }
}

sealed class Truncate_4_1 : Truncate_4 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a1, engine.trail) ) return engine.fail();
        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class Truncate_4_2 : Truncate_4 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        if ( !a2.unify(a4, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class Truncate_4_3 : Truncate_4 {
    static internal readonly IntegerTerm s1 = new IntegerTerm(1);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a5 = ((ListTerm)a1).car;
            a6 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a5 = engine.makeVariable();
            a6 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a5, a6), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isList() ){
            if ( !a5.unify(((ListTerm)a3).car, engine.trail) )
                return engine.fail();
            a7 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a7 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a5, a7), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a8 = engine.makeVariable();
        p1 = new Predicates.Truncate_4(a6, a2, a7, a8, cont);
        return new Predicates.dollar_plus_3(a4, s1, a8, p1);
    }
}
}

