/*
 * *** Please do not edit ! ***
 * @(#) CollapseExtent_2.cs
 * @procedure collapse_extent/2 in mltoprolog.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class CollapseExtent_2 : Predicate, FcoPredicate {
    static internal readonly System.Type EntryCodeType = typeof( CollapseExtent_2 );
    static internal readonly Predicate dollar_fail_0 = new Predicates.dollar_fail_0();
    static internal readonly Predicate CollapseExtent_2_1 = new Predicates.CollapseExtent_2_1();
    static internal readonly Predicate CollapseExtent_2_2 = new Predicates.CollapseExtent_2_2();
    static internal readonly Predicate CollapseExtent_2_var = new Predicates.CollapseExtent_2_var();

    public Term arg1, arg2;

    public CollapseExtent_2(Term a1, Term a2, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        this.cont = cont;
    }

    public CollapseExtent_2(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.SetEntryCode( EntryCodeType, this );
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   CollapseExtent_2_var,
                                   dollar_fail_0,
                                   CollapseExtent_2_1,
                                   dollar_fail_0,
                                   CollapseExtent_2_2
                                   );
    }

    public override int arity() { return 2; }

    public override string ToString() {
        return "collapse_extent(" + arg1 + ", " + arg2 + ")";
    }
}

sealed class CollapseExtent_2_var : CollapseExtent_2 {
    static internal readonly Predicate CollapseExtent_2_var_1 = new Predicates.CollapseExtent_2_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(CollapseExtent_2_1, CollapseExtent_2_var_1);
    }
}

sealed class CollapseExtent_2_var_1 : CollapseExtent_2 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(CollapseExtent_2_2);
    }
}

sealed class CollapseExtent_2_1 : CollapseExtent_2 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("extent", 3);
    static internal readonly IntegerTerm s3 = new IntegerTerm(0);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a1, engine.trail) ) return engine.fail();
        if ( a2.isStructure() ){
            if (f2 != ((StructureTerm)a2).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a2).args;
            a3 = args[0];
            if ( !a3.unify(args[1],  engine.trail) )
                return engine.fail();
            if ( !s3.unify(args[2],  engine.trail) )
                return engine.fail();
        } else if (a2.isVariable() ){
            a3 = engine.makeVariable();
            Term[] args = {a3, a3, s3};
            if ( !a2.unify(new StructureTerm(f2, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.RealInt_2(s3, a3, cont);
        return new Predicates.dollar_neckCut_0(p1);
    }
}

sealed class CollapseExtent_2_2 : CollapseExtent_2 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("extent", 3);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14;
        Predicate p1, p2, p3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a3 = ((ListTerm)a1).car;
            a4 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a3 = engine.makeVariable();
            a4 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a3, a4), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isStructure() ){
            if (f1 != ((StructureTerm)a3).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a3).args;
            a5 = args[0];
            a6 = args[1];
            a7 = args[2];
        } else if (a3.isVariable() ){
            a5 = engine.makeVariable();
            a6 = engine.makeVariable();
            a7 = engine.makeVariable();
            Term[] args = {a5, a6, a7};
            if ( !a3.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a2.isStructure() ){
            if (f1 != ((StructureTerm)a2).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a2).args;
            a8 = args[0];
            a9 = args[1];
            a10 = args[2];
        } else if (a2.isVariable() ){
            a8 = engine.makeVariable();
            a9 = engine.makeVariable();
            a10 = engine.makeVariable();
            Term[] args = {a8, a9, a10};
            if ( !a2.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a12 = engine.makeVariable();
        a13 = engine.makeVariable();
        a14 = engine.makeVariable();
        Term[] h2 = {a12, a13, a14};
        a11 = new StructureTerm(f1, h2);
        p1 = new Predicates.dollar_plus_3(a7, a14, a10, cont);
        p2 = new Predicates.RealMax_3(a6, a13, a9, p1);
        p3 = new Predicates.RealMin_3(a5, a12, a8, p2);
        engine.aregs[1] = a4;
        engine.aregs[2] = a11;
        engine.cont = p3;
        return engine.GetEntryCode( EntryCodeType ).call( engine );
    }
}
}

