/*
 * *** Please do not edit ! ***
 * @(#) Foldr_4.cs
 * @procedure foldr/4 in mltoprolog.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Foldr_4 : Predicate, FcoPredicate {
    static internal readonly System.Type EntryCodeType = typeof( Foldr_4 );
    static internal readonly Predicate Foldr_4_1 = new Predicates.Foldr_4_1();
    static internal readonly Predicate Foldr_4_2 = new Predicates.Foldr_4_2();
    static internal readonly Predicate Foldr_4_sub_1 = new Predicates.Foldr_4_sub_1();

    public Term arg1, arg2, arg3, arg4;

    public Foldr_4(Term a1, Term a2, Term a3, Term a4, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        this.cont = cont;
    }

    public Foldr_4(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.SetEntryCode( EntryCodeType, this );
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(Foldr_4_1, Foldr_4_sub_1);
    }

    public override int arity() { return 4; }

    public override string ToString() {
        return "foldr(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ")";
    }
}

sealed class Foldr_4_sub_1 : Foldr_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Foldr_4_2);
    }
}

sealed class Foldr_4_1 : Foldr_4 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        if ( !a2.unify(a4, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class Foldr_4_2 : Foldr_4 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( a3.isList() ){
            a5 = ((ListTerm)a3).car;
            a6 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a5 = engine.makeVariable();
            a6 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a5, a6), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a7 = engine.makeVariable();
        a8 = engine.makeVariable();
        a12 = new ListTerm(a4, s1);
        a11 = new ListTerm(a7, a12);
        a10 = new ListTerm(a5, a11);
        a9 = new ListTerm(a1, a10);
        p1 = new Predicates.dollar_call_1(a8, cont);
        p2 = new Predicates.eq_stop_stop_2(a8, a9, p1);
        engine.aregs[1] = a1;
        engine.aregs[2] = a2;
        engine.aregs[3] = a6;
        engine.aregs[4] = a7;
        engine.cont = p2;
        return engine.GetEntryCode( EntryCodeType ).call( engine );
    }
}
}

