/*
 * *** Please do not edit ! ***
 * @(#) Merge_3.cs
 * @procedure merge/3 in mltoprolog.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Merge_3 : Predicate {
    static internal readonly Predicate Merge_3_1 = new Predicates.Merge_3_1();
    static internal readonly Predicate Merge_3_2 = new Predicates.Merge_3_2();
    static internal readonly Predicate Merge_3_3 = new Predicates.Merge_3_3();
    static internal readonly Predicate Merge_3_con = new Predicates.Merge_3_con();
    static internal readonly Predicate Merge_3_lis = new Predicates.Merge_3_lis();
    static internal readonly Predicate Merge_3_var = new Predicates.Merge_3_var();

    public Term arg1, arg2, arg3;

    public Merge_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public Merge_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   Merge_3_var,
                                   Merge_3_2,
                                   Merge_3_con,
                                   Merge_3_2,
                                   Merge_3_lis
                                   );
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "merge(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class Merge_3_var : Merge_3 {
    static internal readonly Predicate Merge_3_var_1 = new Predicates.Merge_3_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Merge_3_1, Merge_3_var_1);
    }
}

sealed class Merge_3_var_1 : Merge_3 {
    static internal readonly Predicate Merge_3_var_2 = new Predicates.Merge_3_var_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(Merge_3_2, Merge_3_var_2);
    }
}

sealed class Merge_3_var_2 : Merge_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Merge_3_3);
    }
}

sealed class Merge_3_con : Merge_3 {
    static internal readonly Predicate Merge_3_con_1 = new Predicates.Merge_3_con_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Merge_3_1, Merge_3_con_1);
    }
}

sealed class Merge_3_con_1 : Merge_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Merge_3_2);
    }
}

sealed class Merge_3_lis : Merge_3 {
    static internal readonly Predicate Merge_3_lis_1 = new Predicates.Merge_3_lis_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Merge_3_2, Merge_3_lis_1);
    }
}

sealed class Merge_3_lis_1 : Merge_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Merge_3_3);
    }
}

sealed class Merge_3_1 : Merge_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a1, engine.trail) ) return engine.fail();
        if ( !a2.unify(a3, engine.trail) ) return engine.fail();
        return cont;
    }
}

sealed class Merge_3_2 : Merge_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        if ( !a1.unify(a3, engine.trail) ) return engine.fail();
        return cont;
    }
}

sealed class Merge_3_3 : Merge_3 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("extent", 3);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a4 = ((ListTerm)a1).car;
            a5 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a4 = engine.makeVariable();
            a5 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a4, a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a4.isStructure() ){
            if (f1 != ((StructureTerm)a4).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a4).args;
            a6 = args[0];
            a7 = args[2];
        } else if (a4.isVariable() ){
            a6 = engine.makeVariable();
            a7 = engine.makeVariable();
            Term[] args = {a6, engine.makeVariable(), a7};
            if ( !a4.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a2.isList() ){
            a8 = ((ListTerm)a2).car;
            a9 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a8 = engine.makeVariable();
            a9 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a8, a9), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a8.isStructure() ){
            if (f1 != ((StructureTerm)a8).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a8).args;
            a10 = args[1];
            a11 = args[2];
        } else if (a8.isVariable() ){
            a10 = engine.makeVariable();
            a11 = engine.makeVariable();
            Term[] args = {engine.makeVariable(), a10, a11};
            if ( !a8.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isList() ){
            a12 = ((ListTerm)a3).car;
            a13 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a12 = engine.makeVariable();
            a13 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a12, a13), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a12.isStructure() ){
            if (f1 != ((StructureTerm)a12).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a12).args;
            if ( !a6.unify(args[0],  engine.trail) )
                return engine.fail();
            if ( !a10.unify(args[1],  engine.trail) )
                return engine.fail();
            a14 = args[2];
        } else if (a12.isVariable() ){
            a14 = engine.makeVariable();
            Term[] args = {a6, a10, a14};
            if ( !a12.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.Merge_3(a5, a9, a13, cont);
        return new Predicates.Max_3(a7, a11, a14, p1);
    }
}
}

