import java.util.jar.*;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;

/**
  Build a database of the Java API, in the style of a Prolog database with
  clauses of the following form:
  
  classmember( package, package-class, member-type, member name ).
  interfacemember( package, package, interface, member-type, member name ).
*/

public class BuildDb {

    public static int count = 0;

    public static void processClass( String classname ) throws IOException  {
	// create file in appropriate place to contain the clauses, base location
	// on package name.
	// System.out.println( classname + "\n--------------------------" );
	String packageName;
	classname = classname.replace( '/', '.' );
	int posFirstDollar = classname.indexOf( "$" );
	if( posFirstDollar == -1 ) {
	    packageName = classname.substring( 0, classname.lastIndexOf( '.' ) );
	} else {
	    packageName = classname.substring( 0, posFirstDollar );
	    packageName = packageName.substring( 0, classname.lastIndexOf( '.' ) );
	}
	
	// System.out.println( packageName + "; " + classname );
	File f = new File( "./" + packageName + "_database.pl" );
	RandomAccessFile raf = new RandomAccessFile( f, "rw" );
	raf.seek( raf.length( ) );

	// load the class.
	ClassLoader cl = ClassLoader.getSystemClassLoader( );
	Class c = null;
	try {
	    c = cl.loadClass( classname );
	} catch( ClassNotFoundException _ ) { }

	// iterate thro' the members
	boolean isInterface = c.isInterface( );

	Method[] ms = c.getMethods( );
	Field[] fs = c.getFields( );
	Class superclass = c.getSuperclass( );
	if( superclass != null ) {
	    raf.writeBytes( "derives( '" + superclass.getName( ).replace( '/', '.' ) + "', '" + classname + "' ).\n" );
	    File ff = new File( "./ClassHierarchy.pl" );
	    RandomAccessFile raff = new RandomAccessFile( ff, "rw" );
	    raff.seek( raff.length( ) );
	    raff.writeBytes( "derives( '" + superclass.getName( ).replace( '/', '.' ) + "', '" + classname + "' ).\n" );
	    raff.close( );
	}

	for( int i = 0; i < ms.length; i++ ) {
	    count++;
	    Method m = ms[ i ];
	    boolean isStatic = Modifier.isStatic( m.getModifiers( ) );
	    String stat = isStatic ? "static" : "instance";
	    String name = m.getName( );
	    String sig = "'" + name + "'( ";
	    Class[] partypes = m.getParameterTypes( );
	    String rettype = m.getReturnType( ).getName( );
	    String[] types = new String[ partypes.length + 1 ];
	    types[ 0 ] = rettype;
	    for( int j = 1; j < partypes.length + 1; j++ )
		types[ j ] = partypes[ j - 1 ].getName( );
	    for( int j = 0; j < types.length; j++ ) {
		sig += "'" + types[ j ] + "'";
		if( j == types.length - 1 )
		    sig += " )";
		else
		    sig += ", ";
	    }
	    
	    String clause = null;
	    if( isInterface )
		clause = "interfacemember( ";
	    else
		clause = "classmember( ";
	    clause += "'" + packageName + "', '" + classname + "', " + stat + ", method, " + sig + " ).\n";
	    raf.writeBytes( clause );
	    
	    // System.out.println( name + " " + rettype.toString( ) );
	}

	
	for( int i = 0; i < fs.length; i++ ) {
	    String clause = null;
	    String type = fs[ i ].getType( ).getName( );
	    boolean isStatic = Modifier.isStatic( fs[ i ].getModifiers( ) );
	    String stat = isStatic ? "static" : "instance";
	    if( isInterface )
		clause = "interfacemember( ";
	    else
		clause = "classmember( ";
	    clause += "'" + packageName + "', '" + classname + "', " + stat + ", field, '" + fs[ i ].getName( ) + "'( '" + type + "' ) ).\n";
	    raf.writeBytes( clause );
	    count++;
	}
	
	raf.close( );
    }

    public static void main( String[] args ) {
	/*
	  Package[] ps = Package.getPackages( );
	  for( int i = 0; i < ps.length; i++ ) {
	  String pname = ps[ i ].getName( );
	  System.out.println( pname );
	  }	
	*/
	JarFile j = null;
	try {
	    if( args.length == 0 )
		j = new JarFile( "/usr/java/j2sdk1.4.0_01/jre/lib/rt.jar" );
	    else
		j = new JarFile( args[ 0 ] );
	} catch( IOException e ) { }
	Enumeration e = j.entries( );
	while( e.hasMoreElements( ) ) {
	    Object next = e.nextElement( );
	    String name = next.toString( );
	    if( name.endsWith( ".class" ) ) {
		name = name.substring( 0, name.length( ) - 6 );
	    //System.out.println( name );
		try {
		    processClass( name );
		} catch( IOException ee ) {
		    System.out.println( ee );

		}

	    
	    }
	}
	System.out.println( count );
	
    }
}
