/*
 * *** Please do not edit ! ***
 * @(#) BaseStep_5.cs
 * @procedure base_step/5 in treeify.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class BaseStep_5 : Predicate {
    static internal readonly Predicate dollar_fail_0 = new Predicates.dollar_fail_0();
    static internal readonly Predicate BaseStep_5_1 = new Predicates.BaseStep_5_1();
    static internal readonly Predicate BaseStep_5_2 = new Predicates.BaseStep_5_2();
    static internal readonly Predicate BaseStep_5_var = new Predicates.BaseStep_5_var();

    public Term arg1, arg2, arg3, arg4, arg5;

    public BaseStep_5(Term a1, Term a2, Term a3, Term a4, Term a5, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        arg5 = a5; 
        this.cont = cont;
    }

    public BaseStep_5(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        arg5 = args[4]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.aregs[5] = arg5;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   BaseStep_5_var,
                                   dollar_fail_0,
                                   BaseStep_5_1,
                                   dollar_fail_0,
                                   BaseStep_5_2
                                   );
    }

    public override int arity() { return 5; }

    public override string ToString() {
        return "base_step(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ", " + arg5 + ")";
    }
}

sealed class BaseStep_5_var : BaseStep_5 {
    static internal readonly Predicate BaseStep_5_var_1 = new Predicates.BaseStep_5_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(BaseStep_5_1, BaseStep_5_var_1);
    }
}

sealed class BaseStep_5_var_1 : BaseStep_5 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(BaseStep_5_2);
    }
}

sealed class BaseStep_5_1 : BaseStep_5 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        a5 = engine.aregs[5].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a1, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class BaseStep_5_2 : BaseStep_5 {
    static internal readonly IntegerTerm s1 = new IntegerTerm(1);
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("base", 1);
    static internal readonly SymbolTerm f3 = SymbolTerm.makeSymbol("dbtree", 3);
    static internal readonly SymbolTerm s7 = SymbolTerm.makeSymbol(".");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16;
        Predicate p1, p2, p3, p4, p5, p6, p7, p8;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        a5 = engine.aregs[5].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a6 = ((ListTerm)a1).car;
            a7 = ((ListTerm)a1).cdr;
        } else if ( a1.isVariable() ){
            a6 = engine.makeVariable();
            a7 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a6, a7), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a8 = engine.makeVariable();
        Term[] h4 = {s1, a2, a3};
        a10 = new StructureTerm(f3, h4);
        Term[] h5 = {a10};
        a9 = new StructureTerm(f2, h5);
        Term[] h6 = {s1, a2, a5};
        a11 = new StructureTerm(f3, h6);
        a12 = engine.makeVariable();
        a14 = engine.makeVariable();
        a13 = new ListTerm(engine.makeVariable(), a14);
        a15 = engine.makeVariable();
        a16 = new ListTerm(a12, a14);
        p1 = new Predicates.BaseStep_5(a7, a8, a3, a4, a5, cont);
        p2 = new Predicates.dollar_nl_1(a4, p1);
        p3 = new Predicates.Write_2(a4, s7, p2);
        p4 = new Predicates.WriteSpecial_2(a4, a15, p3);
        p5 = new Predicates.eq_stop_stop_2(a15, a16, p4);
        p6 = new Predicates.eq_stop_stop_2(a6, a13, p5);
        p7 = new Predicates.GetDbtree_2(a11, a12, p6);
        p8 = new Predicates.Assertz_1(a9, p7);
        return new Predicates.dollar_plus_3(a2, s1, a8, p8);
    }
}
}

