/*
 * *** Please do not edit ! ***
 * @(#) GenMasterClauses_3.cs
 * @procedure gen_master_clauses/3 in treeify.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class GenMasterClauses_3 : Predicate {
    static internal readonly Predicate GenMasterClauses_3_1 = new Predicates.GenMasterClauses_3_1();
    static internal readonly Predicate GenMasterClauses_3_2 = new Predicates.GenMasterClauses_3_2();
    static internal readonly Predicate GenMasterClauses_3_sub_1 = new Predicates.GenMasterClauses_3_sub_1();

    public Term arg1, arg2, arg3;

    public GenMasterClauses_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public GenMasterClauses_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(GenMasterClauses_3_1, GenMasterClauses_3_sub_1);
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "gen_master_clauses(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class GenMasterClauses_3_sub_1 : GenMasterClauses_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(GenMasterClauses_3_2);
    }
}

sealed class GenMasterClauses_3_1 : GenMasterClauses_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class GenMasterClauses_3_2 : GenMasterClauses_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol(" :- ");
    static internal readonly SymbolTerm s2 = SymbolTerm.makeSymbol(".");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5;
        Predicate p1, p2, p3, p4, p5;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a3.isList() ){
            a4 = ((ListTerm)a3).car;
            a5 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a4 = engine.makeVariable();
            a5 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a4, a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.GenMasterClauses_3(a1, a2, a5, cont);
        p2 = new Predicates.dollar_nl_1(a1, p1);
        p3 = new Predicates.Write_2(a1, s2, p2);
        p4 = new Predicates.Write_2(a1, a4, p3);
        p5 = new Predicates.Write_2(a1, s1, p4);
        return new Predicates.Write_2(a1, a2, p5);
    }
}
}

