/*
 * *** Please do not edit ! ***
 * @(#) Iterate_4.cs
 * @procedure iterate/4 in treeify.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Iterate_4 : Predicate {
    static internal readonly Predicate Iterate_4_1 = new Predicates.Iterate_4_1();
    static internal readonly Predicate Iterate_4_2 = new Predicates.Iterate_4_2();
    static internal readonly Predicate Iterate_4_var = new Predicates.Iterate_4_var();

    public Term arg1, arg2, arg3, arg4;

    public Iterate_4(Term a1, Term a2, Term a3, Term a4, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        this.cont = cont;
    }

    public Iterate_4(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   Iterate_4_var,
                                   Iterate_4_2,
                                   Iterate_4_2,
                                   Iterate_4_2,
                                   Iterate_4_var
                                   );
    }

    public override int arity() { return 4; }

    public override string ToString() {
        return "iterate(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ")";
    }
}

sealed class Iterate_4_var : Iterate_4 {
    static internal readonly Predicate Iterate_4_var_1 = new Predicates.Iterate_4_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(Iterate_4_1, Iterate_4_var_1);
    }
}

sealed class Iterate_4_var_1 : Iterate_4 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Iterate_4_2);
    }
}

sealed class Iterate_4_1 : Iterate_4 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("dbtree", 3);
    static internal readonly SymbolTerm s4 = SymbolTerm.makeSymbol(" :- ");
    static internal readonly SymbolTerm s5 = SymbolTerm.makeSymbol(".");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6;
        Predicate p1, p2, p3, p4, p5, p6;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        if ( a1.isList() ){
            a5 = ((ListTerm)a1).car;
            if ( !s1.unify(((ListTerm)a1).cdr, engine.trail) )
                return engine.fail();
        } else if ( a1.isVariable() ){
            a5 = engine.makeVariable();
            if ( !a1.unify(new ListTerm(a5, s1), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        Term[] h3 = {engine.makeVariable(), engine.makeVariable(), engine.makeVariable()};
        a6 = new StructureTerm(f2, h3);
        p1 = new Predicates.dollar_nl_1(a4, cont);
        p2 = new Predicates.Write_2(a4, s5, p1);
        p3 = new Predicates.WriteDbtree_2(a4, a5, p2);
        p4 = new Predicates.Write_2(a4, s4, p3);
        p5 = new Predicates.Write_2(a4, a3, p4);
        p6 = new Predicates.dollar_unify_2(a5, a6, p5);
        return new Predicates.dollar_neckCut_0(p6);
    }
}

sealed class Iterate_4_2 : Iterate_4 {
    static internal readonly IntegerTerm s1 = new IntegerTerm(1);
    static internal readonly IntegerTerm s2 = new IntegerTerm(10);
    static internal readonly SymbolTerm s3 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7;
        Predicate p1, p2, p3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        Predicate cont = engine.cont;

        a5 = engine.makeVariable();
        a6 = engine.makeVariable();
        a7 = engine.makeVariable();
        p1 = new Predicates.Iterate_4(a7, a5, a3, a4, cont);
        p2 = new Predicates.IterateThroughBundledList_6(a6, a5, a3, s3, a7, a4, p1);
        p3 = new Predicates.BundleUpOuter_3(a1, a6, s2, p2);
        return new Predicates.dollar_plus_3(a2, s1, a5, p3);
    }
}
}

