/*
 * *** Please do not edit ! ***
 * @(#) PairUp_3.cs
 * @procedure pair_up/3 in treeify.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class PairUp_3 : Predicate {
    static internal readonly Predicate PairUp_3_1 = new Predicates.PairUp_3_1();
    static internal readonly Predicate PairUp_3_2 = new Predicates.PairUp_3_2();
    static internal readonly Predicate PairUp_3_sub_1 = new Predicates.PairUp_3_sub_1();

    public Term arg1, arg2, arg3;

    public PairUp_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public PairUp_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(PairUp_3_1, PairUp_3_sub_1);
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "pair_up(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class PairUp_3_sub_1 : PairUp_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(PairUp_3_2);
    }
}

sealed class PairUp_3_1 : PairUp_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class PairUp_3_2 : PairUp_3 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol(" :- ");
    static internal readonly SymbolTerm s2 = SymbolTerm.makeSymbol(".");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5;
        Predicate p1, p2, p3, p4, p5;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a2.isList() ){
            a4 = ((ListTerm)a2).car;
            a5 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a4 = engine.makeVariable();
            a5 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a4, a5), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.PairUp_3(a1, a5, a3, cont);
        p2 = new Predicates.dollar_nl_1(a3, p1);
        p3 = new Predicates.Write_2(a3, s2, p2);
        p4 = new Predicates.WriteDbtree_2(a3, a4, p3);
        p5 = new Predicates.Write_2(a3, s1, p4);
        return new Predicates.WriteDbtree_2(a3, a1, p5);
    }
}
}

