/*
 * *** Please do not edit ! ***
 * @(#) Treeify_3.cs
 * @procedure treeify/3 in treeify.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Treeify_3 : Predicate {
    static internal readonly Predicate Treeify_3_1 = new Predicates.Treeify_3_1();
    static internal readonly Predicate Treeify_3_2 = new Predicates.Treeify_3_2();
    static internal readonly Predicate Treeify_3_sub_1 = new Predicates.Treeify_3_sub_1();

    public Term arg1, arg2, arg3;

    public Treeify_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public Treeify_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(Treeify_3_1, Treeify_3_sub_1);
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "treeify(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class Treeify_3_sub_1 : Treeify_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Treeify_3_2);
    }
}

sealed class Treeify_3_1 : Treeify_3 {
    static internal readonly IntegerTerm s1 = new IntegerTerm(0);
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("inputclause", 1);
    static internal readonly SymbolTerm s4 = SymbolTerm.makeSymbol("read");
    static internal readonly IntegerTerm s5 = new IntegerTerm(1);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        Term[] h3 = {engine.makeVariable()};
        a4 = new StructureTerm(f2, h3);
        a5 = engine.makeVariable();
        p1 = new Predicates.ReadAndConvertAll_3(a5, a2, s5, cont);
        p2 = new Predicates.Open_3(a1, s4, a5, p1);
        return new Predicates.Retractall_1(a4, p2);
    }
}

sealed class Treeify_3_2 : Treeify_3 {
    static internal readonly IntegerTerm s1 = new IntegerTerm(1);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a3, engine.trail) ) return engine.fail();
        return cont;
    }
}
}

