/*
 * *** Please do not edit ! ***
 * @(#) WriteSpecialArgs_2.cs
 * @procedure write_special_args/2 in treeify.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class WriteSpecialArgs_2 : Predicate {
    static internal readonly Predicate WriteSpecialArgs_2_1 = new Predicates.WriteSpecialArgs_2_1();
    static internal readonly Predicate WriteSpecialArgs_2_2 = new Predicates.WriteSpecialArgs_2_2();
    static internal readonly Predicate WriteSpecialArgs_2_3 = new Predicates.WriteSpecialArgs_2_3();
    static internal readonly Predicate WriteSpecialArgs_2_sub_1 = new Predicates.WriteSpecialArgs_2_sub_1();

    public Term arg1, arg2;

    public WriteSpecialArgs_2(Term a1, Term a2, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        this.cont = cont;
    }

    public WriteSpecialArgs_2(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(WriteSpecialArgs_2_1, WriteSpecialArgs_2_sub_1);
    }

    public override int arity() { return 2; }

    public override string ToString() {
        return "write_special_args(" + arg1 + ", " + arg2 + ")";
    }
}

sealed class WriteSpecialArgs_2_sub_1 : WriteSpecialArgs_2 {
    static internal readonly Predicate WriteSpecialArgs_2_sub_2 = new Predicates.WriteSpecialArgs_2_sub_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(WriteSpecialArgs_2_2, WriteSpecialArgs_2_sub_2);
    }
}

sealed class WriteSpecialArgs_2_sub_2 : WriteSpecialArgs_2 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(WriteSpecialArgs_2_3);
    }
}

sealed class WriteSpecialArgs_2_1 : WriteSpecialArgs_2 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class WriteSpecialArgs_2_2 : WriteSpecialArgs_2 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( a2.isList() ){
            a3 = ((ListTerm)a2).car;
            if ( !s1.unify(((ListTerm)a2).cdr, engine.trail) )
                return engine.fail();
        } else if ( a2.isVariable() ){
            a3 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a3, s1), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.WriteSpecial_2(a1, a3, cont);
        return new Predicates.dollar_neckCut_0(p1);
    }
}

sealed class WriteSpecialArgs_2_3 : WriteSpecialArgs_2 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol(", ");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( a2.isList() ){
            a3 = ((ListTerm)a2).car;
            a4 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a3 = engine.makeVariable();
            a4 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a3, a4), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.WriteSpecialArgs_2(a1, a4, cont);
        p2 = new Predicates.Write_2(a1, s1, p1);
        return new Predicates.WriteSpecial_2(a1, a3, p2);
    }
}
}

