:- dynamic( inputclause/1 ).
:- dynamic( base/1 ).
:- dynamic( pred/1 ).
:- dynamic( subpred/1 ).

% OutputFile does not include the .pl extension
% the files generated will be OutputFile~~1.pl ... OutputFile~~n.pl

treeify( File, OutputFile, 0 ) :-
	retractall( inputclause(X) ),
	open( File, read, In ),
	read_and_convert_all( In, OutputFile, 1 ).
	
treeify( _, _, 1 ).

read_and_convert_all( In, OutputFile, FileNumber ) :-
	read_in_chunk_of_clauses( In ), % fails if file is finished
	once( inputclause( Y ) ),
	generalise_my_vars_no_number( Y, Gen ),
	retractall( pred( X ) ),
	assert( pred( Gen ) ),
    get_output_filename( OutputFile, FileNumber, OutputFileNumber ),
    write( 'output filename: ' ), write( OutputFileNumber ), nl,
    open( OutputFileNumber, write, Out ),
	get_clauses( Clauses ),
    output_chunk( Clauses, Out, FileNumber ),
    close( Out ),
    FileNumberPlus1 is FileNumber + 1,
    read_and_convert_all( In, OutputFile, FileNumberPlus1 ).

read_and_convert_all( File, OutputFile, FileNumber ) :-
    % now generate the master file.  <OutputFile>~~0.pl
    % <base>(...) :- <base>_1(...).
    % ...
    % <base>(...) :- <base>_<FileNumber-1>(...).
    \+( FileNumber < 2 ),
    get_output_filename( OutputFile, 0, OutputFileNumber ),
    open( OutputFileNumber, write, Out ),
	findall( X, subpred( X ), L ),
%	write( L ), nl,
	pred( MasterGen ),
%	write( MasterGen ), nl,
	gen_master_clauses( Out, MasterGen, L ),
	close( Out ).
	
gen_master_clauses( _, _, [] ) :- !.
gen_master_clauses( Out, MasterGen, [H|T] ) :-
	write( Out, MasterGen ),
	write( Out, ' :- ' ),
	write( Out, H ),
	write( Out, '.' ),
	nl( Out ),
	gen_master_clauses( Out, MasterGen, T ).

read_in_chunk_of_clauses( In ) :-
	retractall( inputclause( X ) ),
	read_in_chunk_of_clauses( In, 0, End ),
	End = not_reached_end.

read_in_chunk_of_clauses( In, 500, not_reached_end ) :- !.
read_in_chunk_of_clauses( In, Count, End ) :-
	read( In, X ),
	\+( X == end_of_file ),
	assertz( inputclause( X ) ),
	Count1 is Count + 1,
	read_in_chunk_of_clauses( In, Count1, End ).
read_in_chunk_of_clauses( _, 0, not_reached_end ) :- !.
read_in_chunk_of_clauses( _, _, reached_end ).
	
get_clauses( Clauses ) :-
	findall( X, inputclause( X ), Clauses ).

output_chunk( [H|T], Out, FileNo ) :-
	retractall( base(X) ),
	generalise_my_vars( H, FileNo, Gen ),
	generalise_my_vars_functor( H, FileNo, GenF ),
	assertz( subpred( Gen ) ),
	pred( Pred ),
	'$reverse'( [H|T], Reversed ),
	base_step( Reversed, 1, Gen, Out, GenF ),
	findall( Base, base( Base ), Bases ),
	iterate( Bases, 1, Gen, Out ).

iterate( [X], Count, Gen, Out ) :- !,
	% we have now reached the end of the process.	
	X = dbtree( Count2, _, Clause ),
	write( Out, Gen ),
	write( Out, ' :- ' ),
	write_dbtree( Out, X ),
	write( Out, '.' ),
	nl( Out ).

iterate( List, Count, Gen, Out ) :- 
	Count1 is Count + 1,
	bundle_up_outer( List, BundledList, 10 ),
	iterate_through_bundled_list( BundledList, Count1, Gen, [], NewList, Out ),
	iterate( NewList, Count1, Gen, Out ).

iterate_through_bundled_list( [], _, _, Z, Z, _ ).
iterate_through_bundled_list( [H|T], Count, Gen, Accum, NewList, Out ) :-
	H = [ dbtree( _, First, _ ) | _ ],
	NewElt = dbtree( Count, First, Gen ),
	pair_up( NewElt, H, Out ),
	NewAccum = [NewElt|Accum],
	iterate_through_bundled_list( T, Count, Gen, NewAccum, NewList, Out ).

pair_up( _, [], _ ) :- !.
pair_up( Head, [Body|Bodies], Out ) :-
	write_dbtree( Out, Head ),
	write( Out, ' :- ' ),
	write_dbtree( Out, Body ),
	write( Out, '.' ),
	nl( Out ),
	pair_up( Head, Bodies, Out ).

write_dbtree( Out, dbtree( X, Y, Z ) ) :-
	write( Out, 'dbtree_' ),
	write( Out, X ),
	write( Out, '_' ),
	write( Out, Y ),
	write( Out, '_' ),
	write( Out, Z ).

get_dbtree( dbtree( X, Y, Z ), Structure ) :-
	number_chars( X, Xn ),
	append( "dbtree_", Xn, Inter1 ),
	append( Inter1, "_", Inter2 ),
	number_chars( Y, Yn ),
	append( Inter2, Yn, Inter3 ),
	append( Inter3, "_", Inter4 ),
	atom_chars( Z, Za ),
	append( Inter4, Za, StructureChars ),
	name( Structure, StructureChars ).

base_step( [], _, _, _, _ ) :- !.
base_step( [H|T], Count, Gen, Out, GenF ) :-
	Count1 is Count + 1,
	assertz( base( dbtree( 1, Count, Gen ) ) ),
	get_dbtree( dbtree( 1, Count, GenF ), Functor ),
	H =.. [HFunctor|HArgs],
	ToOutput =.. [Functor|HArgs],
	write_special( Out, ToOutput ),
	write( Out, '.' ),
	nl( Out ),
	base_step( T, Count1, Gen, Out, GenF ).

write_special( Out, ToWrite ) :-
	atom( ToWrite ), !,
	write( Out, '''' ),
	write( Out, ToWrite ),
	write( Out, '''' ).
write_special( Out, ToWrite ) :-
	compound( ToWrite ), !,
	ToWrite =.. [F|Args],
	write( Out, '''' ),
	write( Out, F ),
	write( Out, '''( ' ),
	write_special_args( Out, Args ),
	write( Out, ' )' ).
write_special( Out, ToWrite ) :-
	write( Out, ToWrite ).

write_special_args( Out, [] ) :- !.
write_special_args( Out, [H] ) :- !,
	write_special( Out, H ).
write_special_args( Out, [H|T] ) :-
	write_special( Out, H ),
	write( Out, ', ' ),
	write_special_args( Out, T ).

bundle_up_outer( List, BundledList, BundleSize ) :-
	SizeM1 is BundleSize - 1,
	bundle_up( List, SizeM1, [], BundledList, SizeM1 ).

bundle_up( [], _, Z, Z, _ ).
bundle_up( [H|T], Size, Bundle, Output, Size ) :-
	!,
	bundle_up( T, 0, [ [H] | Bundle ], Output, Size ).
bundle_up( [H|T], Count, [ Hb | Tb ], Output, Size ) :-
	Count1 is Count + 1,
	bundle_up( T, Count1, [ [H|Hb] | Tb ], Output, Size ).

% convert <File> into <File>~~<Number>.pl

get_output_filename( File, Number, FileWithNumber ) :-
	atom_chars( File, FileChars ),
	append( FileChars, "~~", FileChars2 ),
	number_chars( Number, NumChars ),
	append( FileChars2, NumChars, FileWithNumChars ),
	append( FileWithNumChars, ".pl", FileWithPlChars ),
	atom_chars( FileWithNumber, FileWithPlChars ).

% take a structure and munge it into an atom as follows:
% mypred( 1, 'hello', 2, 3 ), 2 |--> 'mypred__2( X1, X2, X3, X4 )'

generalise_my_vars_no_number( Structure, Output ) :-
	Structure =.. [ Functor | Args ],
	atom_chars( Functor, FunctorChars ),
	atom_chars( '( ', LBChars ),
	append( FunctorChars, LBChars, Init ),
	length( Args, NoOfArgs ),
	gen_vars( Init, 1, NoOfArgs, FinalString ), !,
	atom_chars( Output, FinalString ).


generalise_my_vars_functor( Structure, FileNo, Output ) :-
	Structure =.. [ Functor | Args ],
	atom_chars( Functor, FunctorChars0 ),
	number_chars( FileNo, NoChars ),
	append( FunctorChars0, "__", FunctorChars1 ),
	append( FunctorChars1, NoChars, FunctorChars ),
	atom_chars( Output, FunctorChars ).

generalise_my_vars( Structure, FileNo, Output ) :-
	Structure =.. [ Functor | Args ],
	atom_chars( Functor, FunctorChars0 ),
	number_chars( FileNo, NoChars ),
	append( FunctorChars0, "__", FunctorChars1 ),
	append( FunctorChars1, NoChars, FunctorChars ),
	atom_chars( '( ', LBChars ),
	append( FunctorChars, LBChars, Init ),
	length( Args, NoOfArgs ),
	gen_vars( Init, 1, NoOfArgs, FinalString ), !,
	atom_chars( Output, FinalString ).

gen_vars( SoFar, NoOfArgs, NoOfArgs, Output ) :- !,
	atom_chars( 'X', XChars ),
	number_chars( NoOfArgs, NumChars ),
	atom_chars( ' )', RBChars ),
	append( SoFar, XChars, Z1 ),
	append( Z1, NumChars, Z2 ),
	append( Z2, RBChars, Output ).

gen_vars( SoFar, Count, NoOfArgs, Output ) :-
	atom_chars( 'X', XChars ),
	number_chars( Count, NumChars ),
	atom_chars( ', ', CommaChars ),
	append( SoFar, XChars, Z1 ),
	append( Z1, NumChars, Z2 ),
	append( Z2, CommaChars, Out ),
	Count1 is Count + 1,
	gen_vars( Out, Count1, NoOfArgs, Output ).

append( [], L, L ).
append( [H1|T1], L2, [H1|L3] ) :-
	append( T1, L2, L3 ).
