/*
 * *** Please do not edit ! ***
 * @(#) MethArgs_5.cs
 * @procedure meth_args/5 in search.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class MethArgs_5 : Predicate {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("class_method", 4);
    static internal readonly SymbolTerm s2 = SymbolTerm.makeSymbol("method");
    static internal readonly SymbolTerm f3 = SymbolTerm.makeSymbol("list_subset", 2);

    public Term arg1, arg2, arg3, arg4, arg5;

    public MethArgs_5(Term a1, Term a2, Term a3, Term a4, Term a5, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        arg5 = a5; 
        this.cont = cont;
    }

    public MethArgs_5(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        arg5 = args[4]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.setB0();
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11;
        Predicate p1, p2;
        a1 = arg1.dereference();
        a2 = arg2.dereference();
        a3 = arg3.dereference();
        a4 = arg4.dereference();
        a5 = arg5.dereference();

        if ( a5.isStructure() ){
            if (f1 != ((StructureTerm)a5).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a5).args;
            if ( !a2.unify(args[0],  engine.trail) )
                return engine.fail();
            a6 = args[1];
            a7 = args[2];
            if ( !a3.unify(args[3],  engine.trail) )
                return engine.fail();
        } else if (a5.isVariable() ){
            a6 = engine.makeVariable();
            a7 = engine.makeVariable();
            Term[] args = {a2, a6, a7, a3};
            if ( !a5.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a8 = engine.makeVariable();
        a10 = new ListTerm(engine.makeVariable(), a7);
        a9 = new ListTerm(a6, a10);
        Term[] h4 = {a4, a7};
        a11 = new StructureTerm(f3, h4);
        p1 = new Predicates.Once_1(a11, cont);
        p2 = new Predicates.eq_stop_stop_2(a8, a9, p1);
        return new Predicates.Members_5(a1, a2, a3, s2, a8, p2);
    }

    public override int arity() { return 5; }

    public override string ToString() {
        return "meth_args(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ", " + arg5 + ")";
    }
}
}

