/*
 * *** Please do not edit ! ***
 * @(#) PulseList_2.cs
 * @procedure pulse_list/2 in search.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class PulseList_2 : Predicate {
    static internal readonly Predicate PulseList_2_1 = new Predicates.PulseList_2_1();
    static internal readonly Predicate PulseList_2_2 = new Predicates.PulseList_2_2();
    static internal readonly Predicate PulseList_2_sub_1 = new Predicates.PulseList_2_sub_1();

    public Term arg1, arg2;

    public PulseList_2(Term a1, Term a2, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        this.cont = cont;
    }

    public PulseList_2(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(PulseList_2_1, PulseList_2_sub_1);
    }

    public override int arity() { return 2; }

    public override string ToString() {
        return "pulse_list(" + arg1 + ", " + arg2 + ")";
    }
}

sealed class PulseList_2_sub_1 : PulseList_2 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(PulseList_2_2);
    }
}

sealed class PulseList_2_1 : PulseList_2 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("[]");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( !s1.unify(a2, engine.trail) ) return engine.fail();
        return new Predicates.dollar_neckCut_0(cont);
    }
}

sealed class PulseList_2_2 : PulseList_2 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("class", 1);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        Predicate cont = engine.cont;

        if ( a2.isList() ){
            a3 = ((ListTerm)a2).car;
            a4 = ((ListTerm)a2).cdr;
        } else if ( a2.isVariable() ){
            a3 = engine.makeVariable();
            a4 = engine.makeVariable();
            if ( !a2.unify(new ListTerm(a3, a4), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        Term[] h2 = {a3};
        a5 = new StructureTerm(f1, h2);
        p1 = new Predicates.PulseList_2(a1, a4, cont);
        return new Predicates.Pulse_2(a1, a5, p1);
    }
}
}

