/*
 * *** Please do not edit ! ***
 * @(#) Draw2_5.cs
 * @procedure draw2/5 in mltoprolog.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Draw2_5 : Predicate {
    static internal readonly Predicate Draw2_5_1 = new Predicates.Draw2_5_1();
    static internal readonly Predicate Draw2_5_2 = new Predicates.Draw2_5_2();
    static internal readonly Predicate Draw2_5_3 = new Predicates.Draw2_5_3();
    static internal readonly Predicate Draw2_5_sub_1 = new Predicates.Draw2_5_sub_1();

    public Term arg1, arg2, arg3, arg4, arg5;

    public Draw2_5(Term a1, Term a2, Term a3, Term a4, Term a5, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        arg4 = a4; 
        arg5 = a5; 
        this.cont = cont;
    }

    public Draw2_5(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        arg4 = args[3]; 
        arg5 = args[4]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.aregs[4] = arg4;
        engine.aregs[5] = arg5;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(Draw2_5_1, Draw2_5_sub_1);
    }

    public override int arity() { return 5; }

    public override string ToString() {
        return "draw2(" + arg1 + ", " + arg2 + ", " + arg3 + ", " + arg4 + ", " + arg5 + ")";
    }
}

sealed class Draw2_5_sub_1 : Draw2_5 {
    static internal readonly Predicate Draw2_5_sub_2 = new Predicates.Draw2_5_sub_2();

    public override Predicate exec( Prolog engine ) {
        return engine.retry(Draw2_5_2, Draw2_5_sub_2);
    }
}

sealed class Draw2_5_sub_2 : Draw2_5 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Draw2_5_3);
    }
}

sealed class Draw2_5_1 : Draw2_5 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("extent", 3);
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("node", 2);
    static internal readonly SymbolTerm s3 = SymbolTerm.makeSymbol("[]");
    static internal readonly SymbolTerm f4 = SymbolTerm.makeSymbol("pair", 2);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12;
        Predicate p1, p2, p3;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        a5 = engine.aregs[5].dereference();
        Predicate cont = engine.cont;

        if ( a3.isList() ){
            a6 = ((ListTerm)a3).car;
        } else if ( a3.isVariable() ){
            a6 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a6, engine.makeVariable()), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a6.isStructure() ){
            if (f1 != ((StructureTerm)a6).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a6).args;
        } else if (a6.isVariable() ){
            Term[] args = {engine.makeVariable(), engine.makeVariable(), engine.makeVariable()};
            if ( !a6.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a4.isStructure() ){
            if (f2 != ((StructureTerm)a4).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a4).args;
            a7 = args[0];
            if ( !s3.unify(args[1],  engine.trail) )
                return engine.fail();
        } else if (a4.isVariable() ){
            a7 = engine.makeVariable();
            Term[] args = {a7, s3};
            if ( !a4.unify(new StructureTerm(f2, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a7.isStructure() ){
            if (f4 != ((StructureTerm)a7).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a7).args;
            a8 = args[0];
            a9 = args[1];
        } else if (a7.isVariable() ){
            a8 = engine.makeVariable();
            a9 = engine.makeVariable();
            Term[] args = {a8, a9};
            if ( !a7.unify(new StructureTerm(f4, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a5.isStructure() ){
            if (f4 != ((StructureTerm)a5).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a5).args;
            if ( !a8.unify(args[0],  engine.trail) )
                return engine.fail();
            a10 = args[1];
        } else if (a5.isVariable() ){
            a10 = engine.makeVariable();
            Term[] args = {a8, a10};
            if ( !a5.unify(new StructureTerm(f4, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a11 = engine.makeVariable();
        a12 = engine.makeVariable();
        p1 = new Predicates.dollar_cut_1(a11, cont);
        p2 = new Predicates.Drawlabel_4(a8, a12, a2, a10, p1);
        p3 = new Predicates.RealSum_3(a1, a9, a12, p2);
        return new Predicates.dollar_getLevel_1(a11, p3);
    }
}

sealed class Draw2_5_2 : Draw2_5 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("extent", 3);
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("node", 2);
    static internal readonly SymbolTerm f3 = SymbolTerm.makeSymbol("pair", 2);
    static internal readonly IntegerTerm s4 = new IntegerTerm(10);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17, a18, a19, a20, a21, a22, a23, a24;
        Predicate p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        a5 = engine.aregs[5].dereference();
        Predicate cont = engine.cont;

        if ( a3.isList() ){
            a6 = ((ListTerm)a3).car;
            a7 = ((ListTerm)a3).cdr;
        } else if ( a3.isVariable() ){
            a6 = engine.makeVariable();
            a7 = engine.makeVariable();
            if ( !a3.unify(new ListTerm(a6, a7), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a6.isStructure() ){
            if (f1 != ((StructureTerm)a6).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a6).args;
            a8 = args[2];
        } else if (a6.isVariable() ){
            a8 = engine.makeVariable();
            Term[] args = {engine.makeVariable(), engine.makeVariable(), a8};
            if ( !a6.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a4.isStructure() ){
            if (f2 != ((StructureTerm)a4).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a4).args;
            a9 = args[0];
            a10 = args[1];
        } else if (a4.isVariable() ){
            a9 = engine.makeVariable();
            a10 = engine.makeVariable();
            Term[] args = {a9, a10};
            if ( !a4.unify(new StructureTerm(f2, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a9.isStructure() ){
            if (f3 != ((StructureTerm)a9).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a9).args;
            a11 = args[0];
            a12 = args[1];
        } else if (a9.isVariable() ){
            a11 = engine.makeVariable();
            a12 = engine.makeVariable();
            Term[] args = {a11, a12};
            if ( !a9.unify(new StructureTerm(f3, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a5.isList() ){
            a13 = ((ListTerm)a5).car;
        } else if ( a5.isVariable() ){
            a13 = engine.makeVariable();
            if ( !a5.unify(new ListTerm(a13, engine.makeVariable()), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a13.isStructure() ){
            if (f3 != ((StructureTerm)a13).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a13).args;
            if ( !a11.unify(args[0],  engine.trail) )
                return engine.fail();
            a14 = args[1];
        } else if (a13.isVariable() ){
            a14 = engine.makeVariable();
            Term[] args = {a11, a14};
            if ( !a13.unify(new StructureTerm(f3, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        a15 = engine.makeVariable();
        a16 = engine.makeVariable();
        a17 = engine.makeVariable();
        a18 = engine.makeVariable();
        a19 = engine.makeVariable();
        a20 = engine.makeVariable();
        a21 = engine.makeVariable();
        a22 = engine.makeVariable();
        a23 = engine.makeVariable();
        a24 = engine.makeVariable();
        p1 = new Predicates.DoDraw2Map_5(a15, a24, a7, a10, engine.makeVariable(), cont);
        p2 = new Predicates.DoApp_4(a10, a15, a18, a24, p1);
        p3 = new Predicates.Drawline_4(a15, a23, a15, a18, p2);
        p4 = new Predicates.dollar_plus_3(a2, a8, a23, p3);
        p5 = new Predicates.Drawline_4(a20, a18, a22, a18, p4);
        p6 = new Predicates.RealSum_3(a21, a15, a22, p5);
        p7 = new Predicates.Youngestx_2(a10, a21, p6);
        p8 = new Predicates.RealSum_3(a19, a15, a20, p7);
        p9 = new Predicates.Eldestx_2(a10, a19, p8);
        p10 = new Predicates.dollar_plus_3(a17, a16, a18, p9);
        p11 = new Predicates.dollar_plus_3(a2, a8, a17, p10);
        p12 = new Predicates.dollar_unify_2(a16, s4, p11);
        p13 = new Predicates.Drawlabel_4(a11, a15, a2, a14, p12);
        return new Predicates.RealSum_3(a1, a12, a15, p13);
    }
}

sealed class Draw2_5_3 : Draw2_5 {
    static internal readonly SymbolTerm s1 = SymbolTerm.makeSymbol("lock");

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5;
        Predicate p1, p2;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        a4 = engine.aregs[4].dereference();
        a5 = engine.aregs[5].dereference();
        Predicate cont = engine.cont;

        p1 = new Predicates.dollar_abort_0(cont);
        p2 = new Predicates.Unlock_1(s1, p1);
        return new Predicates.dollar_dummyMltoprolog__6_0(p2);
    }
}
}

