/*
 * *** Please do not edit ! ***
 * @(#) Op_3.cs
 * @procedure op/3 in inout.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class Op_3 : Predicate {
    static internal readonly Predicate Op_3_1 = new Predicates.Op_3_1();
    static internal readonly Predicate Op_3_2 = new Predicates.Op_3_2();
    static internal readonly Predicate Op_3_sub_1 = new Predicates.Op_3_sub_1();

    public Term arg1, arg2, arg3;

    public Op_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public Op_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.jtry(Op_3_1, Op_3_sub_1);
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "op(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class Op_3_sub_1 : Op_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(Op_3_2);
    }
}

sealed class Op_3_1 : Op_3 {
    static internal readonly IntegerTerm s1 = new IntegerTerm(0);
    static internal readonly IntegerTerm s2 = new IntegerTerm(1200);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        Predicate p1, p2, p3, p4, p5;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        a4 = engine.makeVariable();
        p1 = new Predicates.dollar_op1_3(a1, a2, a3, cont);
        p2 = new Predicates.dollar_cut_1(a4, p1);
        p3 = new Predicates.dollar_lessOrEqual_2(a1, s2, p2);
        p4 = new Predicates.dollar_lessOrEqual_2(s1, a1, p3);
        p5 = new Predicates.dollar_integer_1(a1, p4);
        return new Predicates.dollar_getLevel_1(a4, p5);
    }
}

sealed class Op_3_2 : Op_3 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol("domain", 2);
    static internal readonly SymbolTerm s2 = SymbolTerm.makeSymbol("integer");
    static internal readonly SymbolTerm f3 = SymbolTerm.makeSymbol("-", 2);
    static internal readonly IntegerTerm s4 = new IntegerTerm(0);
    static internal readonly IntegerTerm s5 = new IntegerTerm(1200);
    static internal readonly Term[] h11 = {s4, s5};
    static internal readonly StructureTerm s6 = new StructureTerm(f3, h11);
    static internal readonly Term[] h12 = {s2, s6};
    static internal readonly StructureTerm s7 = new StructureTerm(f1, h12);
    static internal readonly SymbolTerm f8 = SymbolTerm.makeSymbol("op", 3);
    static internal readonly IntegerTerm s10 = new IntegerTerm(1);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        Term[] h9 = {a1, a2, a3};
        a4 = new StructureTerm(f8, h9);
        return new Predicates.dollar_illarg_3(s7, a4, s10, cont);
    }
}
}

