/*
 * *** Please do not edit ! ***
 * @(#) dollar_transform_3.cs
 * @procedure $transform/3 in plcs.pl
 */

/*
 * @version P# 1.1.2, on Feb 16 2003;  Prolog Cafe 0.44, on November 12 1999
 * @author Mutsunori Banbara (banbara@pascal.seg.kobe-u.ac.jp)
 * @author Naoyuki Tamura    (tamura@kobe-u.ac.jp)
 * Modified by Jonathan Cook (jjc@dcs.ed.ac.uk)
 */
namespace JJC.Psharp.Predicates {

using JJC.Psharp.Lang;
using JJC.Psharp.Lang.Resource;
using Predicates = JJC.Psharp.Predicates;
using Resources = JJC.Psharp.Resources;

public class dollar_transform_3 : Predicate {
    static internal readonly Predicate dollar_transform_3_1 = new Predicates.dollar_transform_3_1();
    static internal readonly Predicate dollar_transform_3_2 = new Predicates.dollar_transform_3_2();
    static internal readonly Predicate dollar_transform_3_var = new Predicates.dollar_transform_3_var();

    public Term arg1, arg2, arg3;

    public dollar_transform_3(Term a1, Term a2, Term a3, Predicate cont) {
        arg1 = a1; 
        arg2 = a2; 
        arg3 = a3; 
        this.cont = cont;
    }

    public dollar_transform_3(){}
    public override void setArgument(Term[] args, Predicate cont) {
        arg1 = args[0]; 
        arg2 = args[1]; 
        arg3 = args[2]; 
        this.cont = cont;
    }

    public override Predicate exec( Prolog engine ) {
        engine.aregs[1] = arg1;
        engine.aregs[2] = arg2;
        engine.aregs[3] = arg3;
        engine.cont = cont;
        return call( engine );
    }

    public virtual Predicate call( Prolog engine ) {
        engine.setB0();
        return engine.switch_on_term(
                                   dollar_transform_3_var,
                                   dollar_transform_3_2,
                                   dollar_transform_3_2,
                                   dollar_transform_3_var,
                                   dollar_transform_3_2
                                   );
    }

    public override int arity() { return 3; }

    public override string ToString() {
        return "$transform(" + arg1 + ", " + arg2 + ", " + arg3 + ")";
    }
}

sealed class dollar_transform_3_var : dollar_transform_3 {
    static internal readonly Predicate dollar_transform_3_var_1 = new Predicates.dollar_transform_3_var_1();

    public override Predicate exec( Prolog engine ) {
        return engine.jtry(dollar_transform_3_1, dollar_transform_3_var_1);
    }
}

sealed class dollar_transform_3_var_1 : dollar_transform_3 {

    public override Predicate exec( Prolog engine ) {
        return engine.trust(dollar_transform_3_2);
    }
}

sealed class dollar_transform_3_1 : dollar_transform_3 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol(":-", 2);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4, a5, a6;
        Predicate p1;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a1.isStructure() ){
            if (f1 != ((StructureTerm)a1).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a1).args;
            a4 = args[0];
            a5 = args[1];
        } else if (a1.isVariable() ){
            a4 = engine.makeVariable();
            a5 = engine.makeVariable();
            Term[] args = {a4, a5};
            if ( !a1.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a3.isStructure() ){
            if (f1 != ((StructureTerm)a3).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a3).args;
            if ( !a4.unify(args[0],  engine.trail) )
                return engine.fail();
            a6 = args[1];
        } else if (a3.isVariable() ){
            a6 = engine.makeVariable();
            Term[] args = {a4, a6};
            if ( !a3.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        p1 = new Predicates.dollar_transformBody_3(a5, a2, a6, cont);
        return new Predicates.dollar_neckCut_0(p1);
    }
}

sealed class dollar_transform_3_2 : dollar_transform_3 {
    static internal readonly SymbolTerm f1 = SymbolTerm.makeSymbol(":-", 2);
    static internal readonly SymbolTerm f2 = SymbolTerm.makeSymbol("true", 1);

    public override Predicate exec( Prolog engine ) {
        Term a1, a2, a3, a4;
        a1 = engine.aregs[1].dereference();
        a2 = engine.aregs[2].dereference();
        a3 = engine.aregs[3].dereference();
        Predicate cont = engine.cont;

        if ( a3.isStructure() ){
            if (f1 != ((StructureTerm)a3).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a3).args;
            if ( !a1.unify(args[0],  engine.trail) )
                return engine.fail();
            a4 = args[1];
        } else if (a3.isVariable() ){
            a4 = engine.makeVariable();
            Term[] args = {a1, a4};
            if ( !a3.unify(new StructureTerm(f1, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        if ( a4.isStructure() ){
            if (f2 != ((StructureTerm)a4).functor )
                return engine.fail();
            Term[] args = ((StructureTerm)a4).args;
            if ( !a2.unify(args[0],  engine.trail) )
                return engine.fail();
        } else if (a4.isVariable() ){
            Term[] args = {a2};
            if ( !a4.unify(new StructureTerm(f2, args), engine.trail) )
                return engine.fail();
        } else {
            return engine.fail();
        }
        return cont;
    }
}
}

