/*
 * @(#)SymbolTerm.cs 
 */

/* 
 * @author Mutsunori Banbara 
 *         banbara@pascal.seg.kobe-u.ac.jp
 *         Nara National College of Technology
 * @author Naoyuki Tamura 
 *         tamura@kobe-u.ac.jp
 *         Kobe University
 */

// import java.util.*;

namespace JJC.Psharp.Lang {

	using System.Collections;

	public sealed class SymbolTerm : Term {

		private static Hashtable symbolTable = 
			Hashtable.Synchronized( new Hashtable( ) );

		private string iname;

	
		public string name {
			get {
				return iname;
			}
			set {
				iname = value;
			}
		}

		private int iarity;

		public int arity {
			get {
				return iarity;
			}
			set {
				iarity = value;
			}
		}

		// f/n$B7?$NA4%j%=!<%9$N(BIndex$B%j%9%H(B
		public Term res = Prolog.Nil;   
		// f/n$B7?$GBh0l0z?t$,JQ?t$N$b$N$N(BIndex$B%j%9%H(B
		public Term res2 = Prolog.Nil;  

		public static void clearRes(){
			SymbolTerm sym = null;
			// IEnumerator e = symbolTable.GetEnumerator( );
			foreach( object symb in symbolTable ) {

				//for (IEnumerator e = symbolTable.GetEnumerator(); e.hasMoreElements() ;) {
				//sym = (SymbolTerm)(e.nextElement());
				sym = (SymbolTerm)symb;
				sym.res  = Prolog.Nil;   
				sym.res2 = Prolog.Nil;
			}
		}

		public static SymbolTerm makeSymbol(string _name){
			return makeSymbol(_name, 0);
		}

		public static SymbolTerm makeSymbol(string _name, int _arity){
			/* TERM EXPERIMENT */
			if( _name.StartsWith( "$" ) )
				_name = "$" + _name.Substring(1);


			/* END EXPERIMENT */
		
			string key = _name + "/" + _arity;
			SymbolTerm sym = (SymbolTerm)symbolTable[key];
			if ( sym == null ){
				sym = new SymbolTerm(_name, _arity);
				symbolTable[ key ] = sym;
			}
			return sym;
		}

		private SymbolTerm(string _name, int _arity) { 
			name  = _name; 
			arity = _arity;
		}

		public override bool unify(Term t, Trail trail) {
			t = t.dereference();
			if ( t.isVariable() )
				return ((VariableTerm)t).unify(this, trail);
			//	return t == this;
			if ( !t.isSymbol() )
				return false;
			return name.Equals(((SymbolTerm)t).name);
		}

		public override bool Equals(Term t) {
			//	t = t.dereference();
			//	return t == this;
			if ( !t.isSymbol() )
				return false;
			return name.Equals(((SymbolTerm)t).name);
		}

		public void undoRes(){
			if ( !res.isList() )
				System.Console.WriteLine("SYSTEM ERROR: in undoRes");
			//	res = ((ListTerm)res).cdr().dereference();
			res = ((ListTerm)res).cdr;
		}

		public void undoRes2(){
			if ( !res2.isList() )
				System.Console.WriteLine("SYSTEM ERROR: in undoRes2");
			//	res2 = ((ListTerm)res2).cdr().dereference();
			res2 = ((ListTerm)res2).cdr;
		}

		public override object toCsObject() {
			return name;
		}

		public override string ToString() { return name; }
	}

}