/*
 *  @(#)PrimRes.cs
 */

/* 
 * @author Mutsunori Banbara 
 *         banbara@pascal.seg.kobe-u.ac.jp
 *         Nara National College of Technology
 * @author Naoyuki Tamura 
 *         tamura@kobe-u.ac.jp
 *         Kobe University
 */

// import java.util.*;

/* Altered by JJC to make the buffer expandable 15/3/2003 */

namespace JJC.Psharp.Lang.Resource {

	using JJC.Psharp.Lang;
	using System.Collections;

	sealed class PrimRes {
		public int level;             // $B%j%=!<%9%l%Y%k(B
		public bool isOutOfScope;  // -<>,=>$B$G$N>CHq2DG=$J%9%3!<%W$N30(B
		public Term head;             // $B%j%=!<%9$N(BHead$BItJ,(B($BFC$KI,MW$J$7(B)
		public SymbolTerm pred;       // functor$B$N%7%s%\%k(B
		public ClosureTerm closure;   // $B%3%s%Q%$%k%3!<%I$X$N;2>H$r<+M3JQ?t$NG[Ns(B
		// more_imp$B$K$h$C$FBeF~$5$l$k(B
		public int from;              // $B4XO"%j%9%H$N3+;OHV9f(B
		public int to;                // $B4XO"%j%9%H$N=*N;HV9f(B
    
		public PrimRes(){}
	}

	/*
	 *  @(#)ResourceTable.cs
	 */

	/* 
	 * @author Mutsunori Banbara 
	 *         banbara@pascal.seg.kobe-u.ac.jp
	 *         Nara National College of Technology
	 * @author Naoyuki Tamura 
	 *         tamura@kobe-u.ac.jp
	 *         Kobe University
	 */

	public sealed class ResourceTable {
		//    protected Prolog engine;
		private Prolog engine;
		//    private Hashtable hash = new Hashtable();
		public readonly Hashtable hash = new Hashtable();
		private int maxContents = 20000;
		private PrimRes[] buffer;

		private int itop;
		public int top {
			get {
				return itop;
			}
			set {
				itop = value;
			}
		}
		public readonly SymbolTerm Nil = SymbolTerm.makeSymbol("[]");
		public readonly SymbolTerm Dot = SymbolTerm.makeSymbol(".");
	
		public ResourceTable(){
			buffer = new PrimRes[maxContents];
			top = 0;
		}
		public ResourceTable(int n){
			maxContents = n;
			buffer = new PrimRes[maxContents];
			top = 0;
		}

		public void setEngine(Prolog _engine){
			engine = _engine;
		}

		public void executeTop(){
			for (int i=0; i<top; i++){
				if ( notConsumed(i) )
					consume(i);
			}	
		}

		public void midWith(){
			for (int i=0; i<top; i++){
				if ( buffer[i].level == engine.U )
					updateLevel(i, engine.L+1);
			}
			engine.L++;
			engine.U++;
		}

		public bool endWith(){
			for (int i=0; i<top; i++){
				if ( notConsumed(i) )
					return false;
			}
			engine.L--;
			return true;
		}

		public void endExpImp(ResIndexTerm _from, ResIndexTerm _to){
			int from = _from.getIndex();
			int to   = _to.getIndex();
			if (from >= to )
				return;
			for (int j=from; j<to; j++){
				buffer[j].isOutOfScope = true;
			}
			engine.trail.push(_from);
			engine.trail.push(_to);
		}

		public bool endImp(ResIndexTerm _from, ResIndexTerm _to){
			int from = _from.getIndex();
			int to   = _to.getIndex();
			if (from >= to )
				return true;
			for (int i=from; i<to; i++){
				if ( notConsumed(i) )
					return false;
			}
			for (int j=from; j<to; j++){
				buffer[j].isOutOfScope = true;
			}
			engine.trail.push(_from);
			engine.trail.push(_to);
			return true;
		}

		private bool notConsumed(int N){
			return (buffer[N].level == engine.L);
		}

		public void setRelatedResField(){
			int from = engine.R0;
			for (int i=from; i<top; i++){
				buffer[i].from = from;
				buffer[i].to   = top-1;
			}
		}

		public void unwind(int i){
			while ( top > i ){
				delete();
			}
		}

		private void delete(){ 
			//	buffer[top--].removeCpfEntry();
			buffer[--top] = null;
		}    

		public bool consumable(SymbolTerm sym, int N){
			return (consumable(N) && buffer[N].pred == sym );
		}

		public bool consumable(int N){
			if ( buffer[N].isOutOfScope )
				return false;
			return ( buffer[N].level == engine.L  || buffer[N].level == 0 );
			//	return ( 0 <= buffer[N].level && buffer[N].level <= engine.L );
		}

		//    public void consume(int M, int N){
		//	if ( ! engine.aregs[M].isResIndex() )
		//	    System.Console.WriteLine("error in consume/2 in resourcetable");
		//	int index = ((ResIndexTerm)(engine.aregs[M])).getIndex();
		//	consume(index);
		//	engine.aregs[N] = buffer[index].closure;
		//    }

		public ClosureTerm getClosureAt(int N){
			return buffer[N].closure;
		}

		public void consume(int N){
			if ( buffer[N].level == 0 )
				return;
			int from = buffer[N].from;
			int to   = buffer[N].to;
			for (int i=from; i<=to; i++){
				updateLevel(i, engine.U);
			}
		}

		public void setLevel(int i, int L){
			buffer[i].level = L;
		}

		public void setOutOfScope(int i, bool L){
			buffer[i].isOutOfScope = L;
		}

		private void updateLevel(int i, int U){
			if ( buffer[i].level == U)
				return;
			engine.trail.push(new ResIndexTerm(i));
			engine.trail.push(new IntegerTerm(buffer[i].level));
			buffer[i].level = U;
		}

		public void setR1andR2(SymbolTerm functor){
			string key;
			Term t;

			//	if ( functor.res  == null) functor.res  = Nil;
			//	if ( functor.res2 == null) functor.res2 = Nil;
			if ( ! functor.res.isList() )
				engine.R1 = engine.R2 = Nil;
			else if ( functor.arity == 0 ){
				key = hashKey(functor);
				t = (Term)hash[key];
				if (t == null) engine.R1 = Nil;
				else engine.R1 = t;
				//	    else engine.R1 = engine.copyTerm(t);
				engine.R2 = Nil;
			}
			else{
				Term arg1 = engine.aregs[1].dereference();
				if ( arg1.isVariable() ){
					//		engine.R1 = engine.copyTerm(functor.res);
					engine.R1 = functor.res;
					engine.R2 = Nil;
				}
				else {
					key = hashOneLevel(functor) + hashOneLevel(arg1);
					t = (Term)hash[key];
					if (t == null) engine.R1 = Nil;
					else engine.R1 = t;
					//		else engine.R1 = engine.copyTerm(t);
					//		engine.R2 = engine.copyTerm(functor.res2);
					engine.R2 = functor.res2;
				}
			}
		}

		private SymbolTerm registerResource(Term _head){
			string key;
			SymbolTerm pred = getFunctor(_head);

			// add hash or res2
			//	if ( pred.res2 == null ) 
			//	    pred.res2 = Nil;
			key = hashKey(_head);
			if ( key == null ){
				pred.res2 = new ListTerm(new ResIndexTerm(top), pred.res2);
				//	    engine.trail.push(pred.res2);
				engine.trail.push(pred);
			} else {
				Term entry = (Term)hash[key];
				if ( entry == null ) 
					entry = Nil;
				entry = new ListTerm(new ResIndexTerm(top), entry);
				hash[ key ] = entry;
				//	    engine.trail.push(entry);
				engine.trail.push(SymbolTerm.makeSymbol(key, -1));
			}
			//	if ( pred.res  == null ) 
			//	    pred.res  = Nil;
			// add res
			pred.res = new ListTerm(new ResIndexTerm(top), pred.res);
			//	engine.trail.push(pred.res);
			engine.trail.push(pred);
			return pred;
		}

		private SymbolTerm getFunctor(Term _head){
			if ( _head.isSymbol() )
				return (SymbolTerm)_head;
			if ( _head.isStructure() )
				return ((StructureTerm)_head).functor;
			System.Console.WriteLine("{TYPE ERROR: Invalid head in getFunctor");
			//	return engine.abort("");
			return null;
		}

		private string hashKey(Term _head){
			Term arg1;

			if ( _head.isVariable() || _head.isClosure() )
				return null;
			else if ( _head.isInteger() || _head.isResIndex() || _head.isSymbol() )
				return _head.ToString();
			else if ( _head.isStructure() )
				arg1 = ((StructureTerm)_head).args[0].dereference();
			else if ( _head.isList() )
				arg1 = ((ListTerm)_head).car.dereference();
			else {
				//return engine.abort("invalid arg in hashkey");
				System.Console.WriteLine("invalid arg in hashkey");
				return null;
			}
			if ( arg1.isVariable() || arg1.isClosure() )
				return null;
			return hashOneLevel(_head) + hashOneLevel(arg1);
		}

		private string hashOneLevel(Term t){
			if ( t.isVariable() || t.isClosure() )
				return null;
			if ( t.isInteger() || t.isResIndex() || t.isSymbol() )
				return t.ToString();
			if ( t.isStructure() )
				return ((StructureTerm)t).functor.ToString();
			if ( t.isList() )
				return Dot.ToString();
			System.Console.WriteLine("error in hashonelevel");
			return null;
		}

		public void addRes(Term _head, ClosureTerm _closure){
			PrimRes r = new PrimRes();
			r.level = engine.L;
			r.isOutOfScope = false;
			r.head = _head;
			r.closure = _closure;
			r.pred  = registerResource(r.head);

			/* START JJC */
			if( top == this.maxContents ) {
				maxContents *= 2;
				PrimRes[] newbuffer = new PrimRes[ maxContents ];
				for( int i = 0; i < top; i++ )
					newbuffer[ i ] = buffer[ i ];
				buffer = newbuffer;
			}
			/* END JJC */
			buffer[top++] = r;
		}
    
		public void addExpRes(Term _head, ClosureTerm _closure){
			PrimRes r = new PrimRes();
			r.level = 0;
			r.isOutOfScope = false;
			r.head = _head;
			r.closure = _closure;
			r.pred  = registerResource(r.head);

			/* START JJC */
			if( top == this.maxContents ) {
				maxContents *= 2;
				PrimRes[] newbuffer = new PrimRes[ maxContents ];
				for( int i = 0; i < top; i++ )
					newbuffer[ i ] = buffer[ i ];
				buffer = newbuffer;
			}
			/* END JJC */
			buffer[top++] = r;
		}

		public void show(){
			for (int i=0; i<top; i++){
				System.Console.Write("res[" + i.ToString() + "]: " + 
					"level = " + buffer[i].level.ToString() + ", " +
					"scope = " + buffer[i].isOutOfScope + ", " +
					"head = " + buffer[i].head.ToString() + ", " +
					"pred = " + buffer[i].pred.name + 
					"/" + 
					buffer[i].pred.arity + ", " +
					"closure = " + buffer[i].closure.ToString() + "\n");
			}
		}
	}

}